// ****************************************************************************
// copyright (c) 2000-2002 Horst Knorr <hk_classes@knoda.org>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file LGPL included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.41 $

#include "hk_kdequery.h"
#include "hk_kdequery.moc"
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qtable.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qmultilineedit.h>
#include <klocale.h>
#include <hk_actionquery.h>
#include <hk_connection.h>
#include "hk_kdegrid.h"
#include <kparts/genericfactory.h>
//#include "hk_kdegridpart.h"
#include <locale.h>

#include <kmenubar.h>
#include <kiconloader.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kstandarddirs.h>

/*
 *  Constructs a hk_kdequery which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 */

hk_kdequery::hk_kdequery(QWidget* w,const char* n,WFlags f):KParts::MainWindow(w,n,f),hk_dsquery()
{
    KIconLoader* loader=KGlobal::iconLoader();
    loader->addAppDir("hk_kdeclasses");

    setXMLFile(locate("data","hk_kdeclasses/hk_kdequery.rc"));

    KLibFactory* p_factory= KLibLoader::self()->factory( "libkatepart" );
    p_katepart = (Kate::Document *) p_factory->create (0L, "kate", "KTextEditor::Document");
    p_designkate=(Kate::View *)p_katepart->createView (this, 0L);
    p_sql_has_changed =false;
    setName( "hk_kdequery" );
    resize( 596, 480 );
    p_factory= KLibLoader::self()->factory( "libhk_kdegridpart" );
    p_part=(KParts::ReadWritePart*) p_factory->create (this, "hk_kdegridpart", "KParts::ReadWritePart");
    p_grid = (hk_kdegrid*)p_part->widget();
    p_grid->set_enablingbehaviour(true,true);
//    p_design = new QMultiLineEdit(this);
    p_qbe = NULL;                                 //new QWidget(this);
    p_autoclose =true;
    p_designaction=new KRadioAction(i18n("&Design mode"),loader->loadIcon("designmode",KIcon::User),0,this,SLOT(designbutton_clicked()),actionCollection(),"designmode");
    p_designaction->setEnabled(!runtime_only());
    p_viewaction=new KRadioAction(i18n("&View mode"),loader->loadIcon("viewmode",KIcon::User),0,this,SLOT(querybutton_clicked()),actionCollection(),"viewmode");
    p_printaction=new KAction(i18n("&Print"),"fileprint",0,this,SLOT(print()),actionCollection(),"print");
//p_printaction->setEnabled(false);
    p_closeaction=new KAction(i18n("&Close"),loader->loadIcon("exit",KIcon::User),0,this,SLOT(close_query()),actionCollection(),"closequery");
    p_saveaction=new KAction(i18n("&Save"),loader->loadIcon("save",KIcon::User),0,this,SLOT(savebutton_clicked()),actionCollection(),"save");
    p_saveasaction=new KAction(i18n("Save &as"),loader->loadIcon("saveas",KIcon::User),0,this,SLOT(saveasbutton_clicked()),actionCollection(),"saveas");
    p_designaction->setExclusiveGroup("mode");
    p_viewaction->setExclusiveGroup("mode");
    set_nodesignmode(runtime_only());
    designbutton_clicked();
    //p_saveaction->setEnabled(false);
    setCentralWidget(p_designkate);
    
    createGUI(p_katepart);
    connect( p_designkate->getDoc(), SIGNAL( textChanged() ), this, SLOT( query_changed() ) );
    p_designkate->setLineNumbersOn(true);
    highlighting_set=false;

}


/*
 *  Destroys the object and frees any allocated resources
 */
hk_kdequery::~hk_kdequery()
{
// no need to delete child widgets, Qt does it all for us
    hk_datasource* d=p_grid->datasource();
    if (d!=NULL)
    {
        if (!d->presentation())d->disable();
    }
    delete p_designaction;
    delete p_viewaction;
    delete p_saveaction;
    delete p_saveasaction;
    delete p_closeaction;

}


void hk_kdequery::set_datasource(hk_datasource* d)
{
    hk_dsmodevisible::set_datasource(d);
    p_grid->set_datasource(d);
    p_designkate->getDoc()->blockSignals(true);
    p_designkate->getDoc()->setText(QString::fromLocal8Bit(d->sql().c_str()));
    p_designkate->getDoc()->blockSignals(false);
    if (d!=NULL)
    {
        d->enable();
    }
    p_sql_has_changed=false;
    //p_saveaction->setEnabled(false);

}


hk_datasource* hk_kdequery::datasource(void)
{

    return hk_dsmodevisible::datasource();
}


void hk_kdequery::designbutton_clicked(void)
{
    if (p_nodesignmode)
    {
        querybutton_clicked();
        return;
    }
    hk_datasource* d=p_grid->datasource();
    if (d!=NULL) d->disable();
    setCentralWidget(p_designkate);
    p_designkate->show();
    p_grid->hide();
    p_designmode=true;
//p_printaction->setEnabled(false);

    p_designaction->setChecked(true);
    createGUI(p_katepart);
    set_caption();
    unsigned int i=0;
    Kate::Document *doc = p_designkate->getDoc();
    if (!highlighting_set)
    {
        while (i<doc->hlModeCount())
        {
            if (doc->hlModeName(i)=="SQL")
            {
                doc->setHlMode(i);
            }
            ++i;
        }
        highlighting_set=true;
    }

}


void hk_kdequery::querybutton_clicked(void)
{
    hk_datasource* d=p_grid->datasource();
    hk_string tst_sql=p_designkate->getDoc()->text().local8Bit().data();
    hk_string separator=" \t\n";
    hk_string::size_type startpos=hk_string::npos;
    for (unsigned int tt=0;tt<tst_sql.size();tt++)
    {
        tst_sql[tt]=toupper(tst_sql[tt]);
    }
   startpos=tst_sql.find_first_not_of(separator);
   if (startpos==hk_string::npos) return;
   hk_string::size_type endpos=tst_sql.find_first_of(separator,startpos+1);
    hk_string value=tst_sql.substr(startpos,endpos-startpos);
    //cerr <<"First word: "<<value<<endl;
    bool actionquery=(value=="ALTER");
    if (!actionquery) actionquery=(value=="DROP");
    if (!actionquery) actionquery=(value=="INSERT");
    if (!actionquery) actionquery=(value=="UPDATE");
    if (!actionquery) actionquery=(value=="CREATE");
    if (!actionquery) actionquery=(value=="GRANT");
    if (!actionquery) actionquery=(value=="REVOKE");
    if (!actionquery) actionquery=(value=="FLUSH");

    if (d!=NULL)
    {
        if (!actionquery)
        {
            d->set_sql(p_designkate->getDoc()->text().local8Bit().data(),true);
            if ( !d->enable())
            {
                hk_string reason=hk_translate("Query could not be executed")+"\n"+hk_translate("Servermessage: ")+d->database()->connection()->last_servermessage();
                show_warningmessage(reason);
                if (!runtime_only())
                {
                    set_sqlmode();
                    return;
                }

            }
        }
        else
        {
            hk_actionquery* q=d->database()->new_actionquery();
            if (q!=NULL)
            {
                hk_string sql=p_designkate->getDoc()->text().local8Bit().data();
                q->set_sql(sql.c_str(),sql.length());
                if (q->execute())
                    show_warningmessage(hk_class::hk_translate("Query was successful"));
                else
                    show_warningmessage(hk_class::hk_translate("Query could not be executed")+"\n"+hk_translate("Servermessage: ")+d->database()->connection()->last_servermessage());
                delete q;
                if (!runtime_only())
                {
                    set_sqlmode();
                    return;
                }
            }

        }
    }
    setCentralWidget(p_grid);
    p_designkate->hide();
    p_grid->show();
//p_printaction->setEnabled(true);

    p_viewaction->setChecked(true);
    p_designmode=false;
    createGUI(p_part);
    set_caption();

}


bool  hk_kdequery::set_mode(enum_mode s)
{
    hk_dsmodevisible::set_mode(s);
    switch (s)
    {
	case hk_dsmodevisible::designmode :
								designbutton_clicked();
								break;
	case hk_dsmodevisible::viewmode :
								querybutton_clicked();
								break;

    }

return true;
}


void hk_kdequery::set_sqlmode(void)
{
    designbutton_clicked();
}


void hk_kdequery::set_viewmode(void)
{
    querybutton_clicked();
}


void hk_kdequery::set_designmode(void)
{
    set_sqlmode();

}


void hk_kdequery::before_source_vanishes(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdequery::before_source_vanishes");
#endif

    if (p_autoclose)
        close();
    else
    {
        hk_datasource* d=datasource();
        if (d!=NULL)d->disable();
        p_grid->set_datasource(NULL);
    }
}


void hk_kdequery::query_changed(void)
{
    p_sql_has_changed=true;
    //p_saveaction->setEnabled(true);

    if (datasource()!=NULL)  datasource()->set_sql(p_designkate->getDoc()->text().local8Bit().data(),true);
}


void hk_kdequery::closeEvent ( QCloseEvent* e)
{
    if (p_sql_has_changed)
    {
        if (p_grid->datasource()!=NULL)  p_grid->save_query();
        p_sql_has_changed=false;

    }
    QWidget::closeEvent(e);

}


void hk_kdequery::set_nodesignmode(bool d)
{
    p_nodesignmode=d;

}


bool hk_kdequery::save_query(const hk_string& n)
{

    set_caption();
    return p_grid->save_query(n);
}


bool hk_kdequery::load_query(void)
{
    bool r= p_grid->load_query();
    hk_datasource* d=datasource();
    if (d==NULL)return false;
    p_designkate->getDoc()->blockSignals(true);
    p_designkate->getDoc()->setText(QString::fromLocal8Bit(d->sql().c_str()));
    p_designkate->getDoc()->blockSignals(false);
	p_grid->set_font(hk_font());
    set_caption();
    return r;
}


void hk_kdequery::savebutton_clicked(void)
{
    if (p_grid->save_query("",false))
    {
        p_sql_has_changed=false;
        //p_saveaction->setEnabled(false);
    }
    set_caption();
}


void hk_kdequery::saveasbutton_clicked(void)
{
    if (!p_grid->datasource()) return;
    hk_string oldname=p_grid->datasource()->name();
    p_grid->datasource()->set_name("");
    if (p_grid->save_query("",false))
    {
        p_sql_has_changed=false;    //p_saveaction->setEnabled(false);

        set_caption();

    }
    else
    {
        p_grid->datasource()->set_name(oldname);

    }

}


void hk_kdequery::close_query(void)
{
    close();
}


void hk_kdequery::set_caption(void)
{
    hk_datasource* d=datasource();
    if (d!=NULL)
    {
        QString n=QString::fromLocal8Bit(d->database()->connection()->drivername().c_str());
        n+=i18n(" Query - ");
        n+=QString::fromLocal8Bit(d->name().c_str());
        n+=" (";
        hk_string driver=d->database()->name();
        n+=QString::fromLocal8Bit(driver.c_str());
        n+=")";
        setCaption(n);
    }

}


void hk_kdequery::print(void)
{

    if (in_designmode())
    {
// this is a workaround: otherwise float numbers in the postscript file
// would be created with locale dots (e.g. a comma in Germany) from Qt
        hk_string origlocale=setlocale(LC_NUMERIC,NULL);
        hk_string origmonetarylocale=setlocale(LC_MONETARY,NULL);
        setlocale(LC_NUMERIC,"C");
        setlocale(LC_MONETARY,"C");
        p_designkate->getDoc()->printDialog();
        cout <<"queryprint"<<endl;
//workaround part 2
        setlocale(LC_NUMERIC,origlocale.c_str());
        setlocale(LC_MONETARY,origmonetarylocale.c_str());
//workaround part 2 end
    }
    else
        p_grid->print_grid();
}


void    hk_kdequery::keyPressEvent ( QKeyEvent * e )
{
  if (p_designmode)
  {
      if (e->state()&ControlButton)
    {
        switch (e->key())
        {
            case Key_C :   
	        {           //copy
		p_designkate->copy();
		} 
                break;
            case Key_V :   //paste
            {
	        p_designkate->paste();
                break;
            }
            case Key_X :   //cut
            {
	       p_designkate->cut();

                break;
            }
	    case Key_Z :
	    {
		p_katepart->undo();
		break;
            }
	    default:;

        }

    } // ende->state()&ControlButton

  }
  KParts::MainWindow::keyPressEvent(e);
}

hk_dsgrid *hk_kdequery::grid(void)
{
return p_grid;
}
